#include "treppe.h"

uint8_t Treppe::softstart_led(uint8_t led, uint16_t startval, uint16_t stopval){
    /*
  softstart task

  - get's called at regular intervals (1ms at the moment)
  - dimms single led (0 - 15, PCA9685 outputs) with linear intervals vom startval to stopval
  - calculates pwm steps depending on startval, stopval and timeinterval
  - -> results in constanst speed
  - returns 1 if led dimming is running
  - returns 0 if led dimming is finished

  */

    static uint8_t lastled = 255;
    static float current_pwm = 0;
    static float stepsize = 0.5;
    if(led != lastled){
      pwmController.setChannelPWM(led, (uint16_t)startval);
      lastled = led;
      current_pwm = startval;
      stepsize = 20*abs(stopval - startval)/(float)time_per_stair;      // only valid at 1ms function call interval
      return 1;
    }
    
    if(current_pwm > stopval - stepsize && current_pwm < stopval + stepsize) return 0;
    // todo: duty cycle zero!
    if(startval > stopval){
      current_pwm -= stepsize;
    }
    else {
      current_pwm += stepsize;
    }
    Serial.println((uint16_t)current_pwm);
    pwmController.setChannelPWM(led, (uint16_t)current_pwm);
    return 1;
}

void Treppe::ledsequence(){
  static int8_t led = 0;
  static uint16_t brightness = 0;
  static uint16_t lastbrightness = 0;
  static uint8_t finish = 1;
  static uint16_t status = 0;
  uint16_t status_build = 0;

  status_build |= direction << 8;
  status_build |= state;
  if(status_build != status){                 // check if any parameter changed
    finish = 0;                               // set state unfinished -> start action
    if(direction) led = 0;                    // reset led counter depending of direction
    else          led = stairs-1;
    if(state){
      brightness = active_brightness;           // set brightness value depending of on/off
      lastbrightness = idle_brightness;
    }     
    else{
      brightness = idle_brightness;
      lastbrightness = active_brightness;
    }          
    status = status_build;                    // set parameter memory
    Serial.print("----Status Changed! onoff: ");
    Serial.print(state);
    Serial.print(" dir: ");
    Serial.println(direction);

  }
  if(!finish){                                // finish == 0 -> action pending
      if(!softstart_led(led,lastbrightness, brightness)){
        Serial.print("one LED finished: ");
        Serial.print(led);
        Serial.print(" last: ");
        Serial.print(lastbrightness);
        Serial.print(" curr: ");
        Serial.println(brightness);
        if(direction){
          led++;
          if(led >= stairs) {
            finish = 1;
            //lastbrightness = brightness;
            }
        } 
        else{
           led--;
           if(led < 0){
             //lastbrightness = brightness;
             finish = 1;
            }
        }
      }
  }
}





void Treppe::elelel()
{
  if(state) {
    if(direction) { // aufwärts
      if(tick >= ticks_treppe-1) { // ziel erreicht
        Serial.println("[Treppe] oberster tick !");
        state = 0;
        return;
      }
      tick++;  // eins hoch
    }
    else {  // abwärts
      if(tick <= 0) { // ziel erreicht
        Serial.println("[Treppe] unterster tick !");
        state = 0;
        return;
      }
      tick--;  // eins runter
    }

    stufe = tick / ticks_pro_stufe;
    float new_pwm = differenz_pwm_pro_tick * (tick - ticks_pro_stufe*stufe);
    if(direction)
      new_pwm += differenz_pwm_pro_tick;
    new_pwm += idle_brightness;

    pwmController.setChannelPWM(stufe, (uint16_t) new_pwm);
    Serial.printf("tick %04u, led %02d:%02u, pwm %4.1f\n", 
      tick,
      stufe,
      (tick - ticks_pro_stufe*stufe), 
      new_pwm
    );
  }
}

// if(stufe > stairs || stufe < 0 || tick < 0 || tick > ticks_treppe-1) {
//   Serial.println("[Treppe] ERROR, Something went wrong !");
//   state = 0;
//   return;
// }

void Treppe::setup(){
    Serial.printf("differenz_pwm_pro_tick %f\n", differenz_pwm_pro_tick);
   
    pwmController.resetDevices();
    
    // Deactivate PCA9685_PhaseBalancer due to LED Flickering
    // https://github.com/NachtRaveVL/PCA9685-Arduino/issues/15
    // see also lib/PCA9685-Arduin/PCA9685.h:204
    pwmController.init(PCA9685_PhaseBalancer_None);
    pwmController.setPWMFrequency(200);
    Serial.println("Hello from Treppe");
    Serial.print("Treppe: initial parameters: stairs=");
    Serial.println(stairs);
    
    for(uint8_t i=0; i<stairs; i++) {
        pwmController.setChannelPWM(i, idle_brightness); 
    }
}

void Treppe::task(){
    //ledsequence();
    task_2();
}

uint16_t Treppe::setIdle(uint16_t _idle_brightness){
    idle_brightness = _idle_brightness;
    Serial.println("Treppe: idle brightness changed!");
    return idle_brightness;
}
uint16_t Treppe::setActive(uint16_t _active_brightness){
    active_brightness = _active_brightness;
    Serial.println("Treppe: active brightness changed!");
    return active_brightness;
}
uint16_t Treppe::setTime(uint16_t _time_per_stair){
    time_per_stair = _time_per_stair;
    Serial.println("Treppe: time changed!");
    return time_per_stair;
}

uint8_t Treppe::setDirection(uint8_t _direction){
    direction = _direction;
    Serial.printf("Treppe: Direction: %d!\n",direction);
    // to do: implement state command variable to determine dimm-state
    return direction;
}

uint8_t Treppe::setState(uint8_t _state){
    if(state == _state) return 1;
    else{
        state = _state;
        Serial.printf("Treppe: State: %d!\n",state);
    }  
    return 0;
}
